import { EventPlugin, PluginType } from '@segment/analytics-react-native';
/**
 * Provides a compatibility layer for LivePlugins to Timeline Plugins
 */
export class LivePluginAsPlugin extends EventPlugin {
  constructor(livePlugin) {
    super();
    this.livePlugin = livePlugin;
    this.type = PluginType[livePlugin.type ?? PluginType.before];
  }
  execute(event) {
    if (this.livePlugin.execute !== undefined) {
      return this.livePlugin.execute(event);
    }
    return super.execute(event);
  }
  track(event) {
    if (this.livePlugin.track !== undefined) {
      return this.livePlugin.track(event);
    }
    return super.track(event);
  }
  identify(event) {
    if (this.livePlugin.identify !== undefined) {
      return this.livePlugin.identify(event);
    }
    return super.identify(event);
  }
  group(event) {
    if (this.livePlugin.group !== undefined) {
      return this.livePlugin.group(event);
    }
    return super.group(event);
  }
  alias(event) {
    if (this.livePlugin.alias !== undefined) {
      return this.livePlugin.alias(event);
    }
    return super.alias(event);
  }
  screen(event) {
    if (this.livePlugin.screen !== undefined) {
      return this.livePlugin.screen(event);
    }
    return super.screen(event);
  }
  reset() {
    this.livePlugin.reset?.();
  }
  flush() {
    this.livePlugin.flush?.();
  }
}
export class SegmentClientLivePluginLayer {
  /**
   * Unsubscribe function for new plugin listener
   */

  constructor(analytics) {
    this.analytics = analytics;
  }

  /*
      Get the traits for the current identified user
      Returns: 
          Object/Dictionary containing the users traits
      */
  get traits() {
    return this.analytics.userInfo.get().traits ?? {};
  }

  /*
      Get the current user ID
      */
  get userId() {
    return this.analytics.userInfo.get().userId;
  }

  /*
      Get the current anonymous ID
      */
  get anonymousId() {
    return this.analytics.userInfo.get().anonymousId;
  }

  /*
      Send a track event to segment
      Params:
          event: String - the name of the event
          properties: Object - any additional properties or null
      */
  track(event, properties) {
    return this.analytics.track(event, properties, e => {
      return this.insertEventOrigin(e);
    });
  }

  /*
      Send an identify event to segment
      Params:
          userId: String - the user id
          traits: Object - any additional information about the user
      */
  identify(userId, traits) {
    return this.analytics.identify(userId, traits, e => {
      return this.insertEventOrigin(e);
    });
  }

  /*
      Send a screen event to segment
      Params:
          title: String - The title of this screen
          category: String - The category of this screen, or null
          properties: Object - Any additional properties or null
      */
  screen(title, _category, properties) {
    return this.analytics.screen(title, properties, e => {
      return this.insertEventOrigin(e);
    });
  }

  /*
      Send a group event to segment
      Params:
          groupId: String - The groupId to associate with this session
          traits: Object - Any additional information about this group or session
      */
  group(groupId, traits) {
    return this.analytics.group(groupId, traits, e => {
      return this.insertEventOrigin(e);
    });
  }

  /*
      Send an alias event to segment
      Params:
          newId: String - Associate this new ID with this user as an alias.
      */
  alias(newId) {
    return this.analytics.alias(newId, e => {
      return this.insertEventOrigin(e);
    });
  }

  /*
      Resets the current session.  A new anonymousId is created.  All user identity,
      traits, etc. are removed from this analytics instance.
      */
  reset() {
    return this.analytics.reset();
  }

  /*
      Tell this analytics instance to send any queued events off to segment.
      */
  flush() {
    return this.analytics.flush();
  }

  /*
      Add an LivePlugin subclass to this analytics instance. The LivePlugin subclass will
      determine whether this LivePlugin should run as part of the soure timeline, or
      a specific destination timeline.
      */
  add(livePlugin) {
    // Create a plugin layer to enable this livePlugin as a plugin in the timeline
    const livePluginPlugin = new LivePluginAsPlugin(livePlugin);
    const destination = livePlugin.destination;
    if (destination !== undefined && destination !== null) {
      // Find the destination in the timeline and add it there
      this.addDestinationFunction(livePluginPlugin);
      // Also subscribe to any new plugins so that they also get the right liveplugin added right away
      this.unsubscribePluginListener = this.analytics.onPluginLoaded(addedPlugin => {
        if (addedPlugin.type === PluginType.destination) {
          this.addToDestination(addedPlugin, livePluginPlugin);
        }
      });
    } else {
      this.analytics.add({
        plugin: livePluginPlugin
      });
    }
  }
  addDestinationFunction(plugin) {
    const key = plugin.livePlugin.destination;
    if (key === undefined) {
      return;
    }
    this.analytics.getPlugins(PluginType.destination).forEach(destination => {
      this.addToDestination(destination, plugin);
    });
  }
  addToDestination(destination, livePlugin) {
    if (livePlugin.livePlugin.destination === undefined || destination === null || destination === undefined) {
      return;
    }
    if (destination.key === livePlugin.livePlugin.destination) {
      destination.add(livePlugin);
    }
  }

  /**
   * Cleans and removes all background listeners
   */
  cleanup() {
    this.unsubscribePluginListener?.();
  }
  insertEventOrigin(event) {
    if (!event.context) event.context = {};
    if (!event.context.__eventOrigin) event.context.__eventOrigin = {};
    event.context.__eventOrigin.type = 'js';
    return event;
  }
}
//# sourceMappingURL=compat.js.map