import {
  AliasEventType,
  DestinationPlugin,
  EventPlugin,
  GroupEventType,
  GroupTraits,
  IdentifyEventType,
  JsonMap,
  PluginType,
  ScreenEventType,
  SegmentClient,
  SegmentEvent,
  TrackEventType,
  UserTraits,
} from '@segment/analytics-react-native';
import type { LivePlugin } from './types';

/**
 * Provides a compatibility layer for LivePlugins to Timeline Plugins
 */
export class LivePluginAsPlugin extends EventPlugin {
  readonly livePlugin: LivePlugin;

  constructor(livePlugin: LivePlugin) {
    super();
    this.livePlugin = livePlugin;
    this.type = PluginType[livePlugin.type ?? PluginType.before];
  }

  execute(event: SegmentEvent) {
    if (this.livePlugin.execute !== undefined) {
      return this.livePlugin.execute(event);
    }
    return super.execute(event);
  }

  track(event: TrackEventType) {
    if (this.livePlugin.track !== undefined) {
      return this.livePlugin.track(event) as TrackEventType;
    }
    return super.track(event);
  }

  identify(event: IdentifyEventType) {
    if (this.livePlugin.identify !== undefined) {
      return this.livePlugin.identify(event) as IdentifyEventType;
    }
    return super.identify(event);
  }

  group(event: GroupEventType) {
    if (this.livePlugin.group !== undefined) {
      return this.livePlugin.group(event) as GroupEventType;
    }
    return super.group(event);
  }

  alias(event: AliasEventType) {
    if (this.livePlugin.alias !== undefined) {
      return this.livePlugin.alias(event) as AliasEventType;
    }
    return super.alias(event);
  }

  screen(event: ScreenEventType) {
    if (this.livePlugin.screen !== undefined) {
      return this.livePlugin.screen(event) as ScreenEventType;
    }
    return super.screen(event);
  }

  reset() {
    this.livePlugin.reset?.();
  }

  flush() {
    this.livePlugin.flush?.();
  }
}

export class SegmentClientLivePluginLayer {
  private analytics: SegmentClient;
  /**
   * Unsubscribe function for new plugin listener
   */
  private unsubscribePluginListener?: () => void;

  constructor(analytics: SegmentClient) {
    this.analytics = analytics;
  }

  /*
      Get the traits for the current identified user
      Returns: 
          Object/Dictionary containing the users traits
      */
  get traits(): UserTraits | GroupTraits {
    return this.analytics.userInfo.get().traits ?? {};
  }

  /*
      Get the current user ID
      */
  get userId(): string | undefined {
    return this.analytics.userInfo.get().userId;
  }

  /*
      Get the current anonymous ID
      */
  get anonymousId(): string {
    return this.analytics.userInfo.get().anonymousId;
  }

  /*
      Send a track event to segment
      Params:
          event: String - the name of the event
          properties: Object - any additional properties or null
      */
  track(event: string, properties?: JsonMap) {
    return this.analytics.track(event, properties, (e) => {return this.insertEventOrigin(e)});
  }

  /*
      Send an identify event to segment
      Params:
          userId: String - the user id
          traits: Object - any additional information about the user
      */
  identify(userId?: string, traits?: UserTraits) {
    return this.analytics.identify(userId, traits, (e) => {return this.insertEventOrigin(e)});
  }

  /*
      Send a screen event to segment
      Params:
          title: String - The title of this screen
          category: String - The category of this screen, or null
          properties: Object - Any additional properties or null
      */
  screen(title: string, _category?: string, properties?: JsonMap) {
    return this.analytics.screen(title, properties, (e) => {return this.insertEventOrigin(e)});
  }

  /*
      Send a group event to segment
      Params:
          groupId: String - The groupId to associate with this session
          traits: Object - Any additional information about this group or session
      */
  group(groupId: string, traits?: GroupTraits) {
    return this.analytics.group(groupId, traits, (e) => {return this.insertEventOrigin(e)});
  }

  /*
      Send an alias event to segment
      Params:
          newId: String - Associate this new ID with this user as an alias.
      */
  alias(newId: string) {
    return this.analytics.alias(newId, (e) => {return this.insertEventOrigin(e)});
  }

  /*
      Resets the current session.  A new anonymousId is created.  All user identity,
      traits, etc. are removed from this analytics instance.
      */
  reset() {
    return this.analytics.reset();
  }

  /*
      Tell this analytics instance to send any queued events off to segment.
      */
  flush() {
    return this.analytics.flush();
  }

  /*
      Add an LivePlugin subclass to this analytics instance. The LivePlugin subclass will
      determine whether this LivePlugin should run as part of the soure timeline, or
      a specific destination timeline.
      */
  add(livePlugin: LivePlugin) {
    // Create a plugin layer to enable this livePlugin as a plugin in the timeline
    const livePluginPlugin = new LivePluginAsPlugin(livePlugin);
    const destination = livePlugin.destination;
    if (destination !== undefined && destination !== null) {
      // Find the destination in the timeline and add it there
      this.addDestinationFunction(livePluginPlugin);
      // Also subscribe to any new plugins so that they also get the right liveplugin added right away
      this.unsubscribePluginListener = this.analytics.onPluginLoaded(
        (addedPlugin) => {
          if (addedPlugin.type === PluginType.destination) {
            this.addToDestination(
              addedPlugin as DestinationPlugin,
              livePluginPlugin
            );
          }
        }
      );
    } else {
      this.analytics.add({ plugin: livePluginPlugin });
    }
  }

  private addDestinationFunction(plugin: LivePluginAsPlugin) {
    const key = plugin.livePlugin.destination;

    if (key === undefined) {
      return;
    }

    this.analytics.getPlugins(PluginType.destination).forEach((destination) => {
      this.addToDestination(destination as DestinationPlugin, plugin);
    });
  }

  private addToDestination(
    destination: DestinationPlugin,
    livePlugin: LivePluginAsPlugin
  ) {
    if (
      livePlugin.livePlugin.destination === undefined ||
      destination === null ||
      destination === undefined
    ) {
      return;
    }

    if (destination.key === livePlugin.livePlugin.destination) {
      destination.add(livePlugin);
    }
  }

  /**
   * Cleans and removes all background listeners
   */
  cleanup() {
    this.unsubscribePluginListener?.();
  }

  private insertEventOrigin(event: SegmentEvent) : SegmentEvent {
    if (!event.context) event.context = {};
    if (!event.context.__eventOrigin) event.context.__eventOrigin = {};
    event.context.__eventOrigin.type = 'js';
    return event;
  }
}
