import {
  PluginType,
  SegmentAPISettings,
  SegmentClient,
  UpdateType,
  UtilityPlugin,
  EdgeFunctionSettings,
} from '@segment/analytics-react-native';
import { Store } from '@segment/sovran-react-native';
import { SegmentClientLivePluginLayer } from './compat';
import { sandboxExecute } from './sandbox';
import * as RNFS from 'react-native-fs';
import { extractFunctionNames } from './util';

export interface LivePluginDependent {
  prepare(compatLayer: SegmentClientLivePluginLayer): void;
  readyToStart(): void;
}

const hasEdgeFunction = (
  settings?: EdgeFunctionSettings | {}
): settings is EdgeFunctionSettings =>
  settings !== undefined &&
  settings !== null &&
  Object.keys(settings).length > 0;

interface SettingsStore {
  edgeFunction?: EdgeFunctionSettings;
  currentVersion?: string;
}

const FILE_PATH = RNFS.DocumentDirectoryPath + '/liveplugin.js';

export class LivePlugins extends UtilityPlugin {
  type = PluginType.before;
  private compatLayer?: SegmentClientLivePluginLayer;
  private store: Store<SettingsStore> | undefined;
  private dependents: LivePluginDependent[] = [];
  static loaded = false;

  loadEdgeFunction = async (settings: EdgeFunctionSettings) => {
    if (this.analytics === undefined) {
      return;
    }

    if (!hasEdgeFunction(settings)) {
      return;
    }

    const hasFile = await RNFS.exists(FILE_PATH);
    if (!hasFile) {
      this.analytics?.logger.info(
        'Edge Function has not been downloaded yet. Skipping until settings update'
      );
      return;
    }

    try {
      let js = await RNFS.readFile(FILE_PATH);

      this.compatLayer = new SegmentClientLivePluginLayer(this.analytics);
      this.dependents.forEach((d) => {
        d.prepare(this.compatLayer!);
      });

      try {
        // extract functions defined in the js code
        // and attach it to global, so we may call it
        // via the function name
        const functions = extractFunctionNames(js);
        functions.forEach((f) => {
          js += '\nglobal.' + f + ' = ' + f + ';';
        })
        
        sandboxExecute(this.compatLayer, js);
        this.dependents.forEach((d) => {
          d.readyToStart();
        })
      } catch (e) {
        this.analytics.logger?.warn('Edge Function error:', e);
        // If the liveplugin fails to execute we won't update the version we're running,
        return;
      }

      await this.store?.dispatch((state) => {
        return {
          ...state,
          currentVersion: state.edgeFunction?.version,
        };
      });
    } catch (e) {
      this.analytics?.logger.warn(
        `LivePlugin couldn't be loaded: ${JSON.stringify(e)}`
      );
    }
  };

  async downloadEdgeFunction(downloadURL: string, version: string) {
    if (this.analytics === undefined) {
      return;
    }

    try {
      // Download the file
      // 'https://cdn.liveplugin.segment.com/3psnN9oWENuLqmfAtKLcKh/e812b97d-6d21-408f-8533-9582fa8b0ed7.js'
      const response = await fetch(downloadURL);
      const js = await response.text();

      await RNFS.writeFile(FILE_PATH, js);

      await this.store?.dispatch((state) => {
        return {
          ...state,
          settings: {
            downloadURL,
            version,
          },
        };
      });
      this.analytics.logger.info(
        `LivePlugin version ${version} downloaded. It will apply after the next app startup`
      );

      return version;
    } catch (e) {
      this.analytics.logger.warn(
        `LivePlugin version ${version} failed to download: ${JSON.stringify(e)}`
      );
    }
    return;
  }

  override update(_: SegmentAPISettings, _type: UpdateType): void {
    if (LivePlugins.loaded) return;
    LivePlugins.loaded = true;
    
    const edgeFunctionSettings = this.analytics?.edgeFunctionSettings.get();
    if (hasEdgeFunction(edgeFunctionSettings)) {
      const config = edgeFunctionSettings;
      const currentVersion = this.store?.getState().currentVersion;

      if (config.version !== currentVersion) {
        this.analytics?.logger.info(
          `LivePlugin version ${config.version} found. Currently running version ${currentVersion}.`
        );
        this.downloadEdgeFunction(config.downloadURL, config.version);
      }
    }
  }

  override configure(analytics: SegmentClient): void {
    this.analytics = analytics;

    const edgeFunctionSettings = analytics?.edgeFunctionSettings.get();
    if (
      typeof edgeFunctionSettings === 'object' &&
      edgeFunctionSettings != null
    ) {
      this.loadEdgeFunction(edgeFunctionSettings).catch((error) => {
        console.error('Error when download edge function:', error);
      });
    }
  }

  shutdown(): void {
    this.compatLayer?.cleanup();
  }

  addDependent(plugin: LivePluginDependent) {
    this.dependents.push(plugin);
    if (LivePlugins.loaded) {
      plugin.prepare(this.compatLayer!);
      plugin.readyToStart();
    }
  }
}
