/* eslint-disable no-new-func */

import type { SegmentClientLivePluginLayer } from './compat';
import { LivePluginBase, LivePluginType } from './types';

function isObject(obj: unknown): obj is Object {
  return typeof obj === 'object' && !Array.isArray(obj) && obj !== null;
}

/**
 * SafeProxy prevents the caller from escaping the sandbox through function, objects, classes
 * Blocks any access to prototype and extensibility (but the user is free to declare new objects, extend classes or call functions)
 *
 * It wraps inner properties in the same SafeProxy to prevent leaking through inner objects
 *
 */
export class SafeProxy<T extends object> implements ProxyHandler<T> {
  // Getting prototype is allowed
  getPrototypeOf(target: T): object | null {
    return Reflect.getPrototypeOf(target);
  }
  // Setting prototypes is not allowed
  setPrototypeOf(_target: T, _value: object | null) {
    return false;
  }

  defineProperty(
    _target: T,
    _property: string | symbol,
    _attributes: PropertyDescriptor
  ): boolean {
    return false;
  }

  // No extending the objects
  preventExtensions(_target: T): boolean {
    return true;
  }
  isExtensible(_target: T): boolean {
    return false;
  }

  // All inner objects will get wrapped by the same proxy
  get(target: T, prop: string | symbol, receiver: any): any {
    // No Prototype access
    if (prop === 'prototype' || prop === '__proto__') {
      return Reflect.get(target, prop, receiver);
    }
    // No building new objects out of existing objects
    if (typeof target === 'object' && prop === 'constructor') {
      return null;
    }
    // No accessing the call stack information
    if (prop === 'arguments' || prop === 'caller' || prop === 'callee') {
      return null;
    }
    // TODO: Fix for non-configurable, non-writeable properties
    const propValue = Reflect.get(target, prop, receiver);
    // Inner object props are wrapped with the same proxy
    if (isObject(propValue)) {
      // We only need to wrap configurable/extensible properties, as the rest are already read only
      const isConfigurable = Object.isExtensible(propValue);
      const isFrozen = Object.isFrozen(propValue);
      const isSealed = Object.isSealed(propValue);

      if (isConfigurable && !isFrozen && !isSealed) {
        return new Proxy(propValue, new SafeProxy());
      }
    }
    return Reflect.get(target, prop, receiver);
  }
  set(target: T, prop: string | symbol, newValue: any, receiver: any): boolean {
    if (
      ['__proto__', 'prototype', 'arguments', 'caller', 'callee'].includes(
        prop as string
      )
    ) {
      return false;
    }

    return Reflect.set(target, prop, newValue, receiver);
  }
}

/**
 * This function limits the scope of the LivePlugin as most as we can so that they
 * don't pollute our global scope. It clousures only the stuff we want to show
 * to the eval function
 * 
 * Use `paramsKeyValue` to export more objects to eval in the format of:
 * { '<alias name>' : <actual instance of the object>}
 *
 * @throws
 *
 * @returns code execution result (if any)
 */
export function sandboxExecute(
  analytics: SegmentClientLivePluginLayer,
  livePluginCode: string,
  paramsKeyValue?: {}
) {
  const paramNames = Object.keys(paramsKeyValue ?? {});
  const paramInstances = Object.values(paramsKeyValue ?? {});

  'use strict';
  const func = new Function(
    'analytics',
    'LivePlugin',
    'LivePluginType',
    // No access to any of these
    'self',
    'alert',
    'window',
    'global',
    'fetch',
    'XMLHttpRequest',
    'setTimeout',
    'setInterval',
    ...paramNames,
    // And our function code
    `'use strict'; return (() => { ${livePluginCode} })();`
  );

  const result = func(
    // analytics client
    new Proxy(analytics, new SafeProxy()),
    // LivePlugin class
    new Proxy(LivePluginBase, new SafeProxy()),
    // LivePluginType enum
    new Proxy(LivePluginType, new SafeProxy()),
    null, // self
    null, // alert
    null, // window
    global, // global
    null, // fetch
    null, // XMLHTTPRequest
    null, //setTimeout
    null, //setInterval
    ...paramInstances
  );
  return result;
}
