/* eslint-disable @typescript-eslint/no-unused-vars */
import {
  type SegmentAPISettings,
  type UpdateType,
  type SegmentEvent,
  EventType,
} from '@segment/analytics-react-native';

// Represents an enum defining an edge function type
export enum LivePluginType {
  before = 'before',
  enrichment = 'enrichment',
  after = 'after',
  utility = 'utility',
}

export interface LivePlugin {
  readonly type?: LivePluginType;
  readonly destination?: string;

  update?(settings: SegmentAPISettings, type: UpdateType): void;
  execute?(event: SegmentEvent): SegmentEvent | undefined;
  track?(event: SegmentEvent): SegmentEvent | undefined;
  identify?(event: SegmentEvent): SegmentEvent | undefined;
  screen?(event: SegmentEvent): SegmentEvent | undefined;
  alias?(event: SegmentEvent): SegmentEvent | undefined;
  group?(event: SegmentEvent): SegmentEvent | undefined;
  flush?(): void;
  reset?(): void;
}

// Subclass LivePlugin to add custom behaviors/transformations
export abstract class LivePluginBase implements LivePlugin {
  private _type: LivePluginType;
  private _destination: string;

  /* 
      Create an Edge Function
      Params:
          type: LivePluginType - The type of edge function
          destination: String - A string representing the destination key this
                                edge function will be active for
      */
  constructor(type: LivePluginType, destination: string) {
    this._type = type;
    this._destination = destination;
  }

  /*
      Returns the LivePluginType designation for this edge function
      */
  get type() {
    return this._type;
  }

  /*
      Returns the destination this edge function is active on, or null
      */
  get destination() {
    return this._destination;
  }

  /*
      Update is called by the system when we receive new settings from Segment
      or the Analytics system is starting up.  
      
      Implementing this method is useful
      for doing any setup that might be needed based on settings.
      
      Params:
          settings: Object - A key/value object of all settings received from Segment
          type: UpdateType - Determines if this is an initial batch of settings or a
                             subsequent update
      */
  // @ts-ignore

  update(settings: SegmentAPISettings, type: UpdateType): void {
    // do nothing by default, user can override.
  }

  /*
      When events move through the system, `execute` is called.  This will naturally 
      call out to the appropriate handlers such as `track`, `identify`, etc.  If you
      want to perform transformations regardless of event type, override this method.
      
      Call `super.execute()` if you'd like to preserve existing behavior of the base
      class.
      
      Params:
          event: Object - A key/value set of event data.
      
      Returns: 
          A transformed or modified event object, the original event object,
          or null if the event is to be discarded.
      */
  execute(event: SegmentEvent): SegmentEvent | undefined {
    if (event === undefined) {
      return undefined;
    }

    let result: Promise<SegmentEvent | undefined> | SegmentEvent | undefined =
      event;
    switch (result.type) {
      case EventType.IdentifyEvent:
        result = this.identify(result);
        break;
      case EventType.TrackEvent:
        result = this.track(result);
        break;
      case EventType.ScreenEvent:
        result = this.screen(result);
        break;
      case EventType.AliasEvent:
        result = this.alias(result);
        break;
      case EventType.GroupEvent:
        result = this.group(result);
        break;
    }
    return result;
  }

  /*
      Override this method to apply transformations specific to track events.
      
      Params:
          event: Object - A key/value set of event data.
      
      Returns: 
          A transformed or modified event object, the original event object,
          or null if the event is to be discarded.
      */
  track(event: SegmentEvent): SegmentEvent | undefined {
    return event;
  }

  /*
      Override this method to apply transformations specific to identify events.
      
      Params:
          event: Object - A key/value set of event data.
      
      Returns: 
          A transformed or modified event object, the original event object,
          or null if the event is to be discarded.
      */
  identify(event: SegmentEvent): SegmentEvent | undefined {
    return event;
  }

  /*
      Override this method to apply transformations specific to group events.
      
      Params:
          event: Object - A key/value set of event data.
      
      Returns: 
          A transformed or modified event object, the original event object,
          or null if the event is to be discarded.
      */
  group(event: SegmentEvent): SegmentEvent | undefined {
    return event;
  }

  /*
      Override this method to apply transformations specific to alias events.
      
      Params:
          event: Object - A key/value set of event data.
      
      Returns: 
          A transformed or modified event object, the original event object,
          or null if the event is to be discarded.
      */
  alias(event: SegmentEvent): SegmentEvent | undefined {
    return event;
  }

  /*
      Override this method to apply transformations specific to screen events.
      
      Params:
          event: Object - A key/value set of event data.
      
      Returns: 
          A transformed or modified event object, the original event object,
          or null if the event is to be discarded.
      */
  screen(event: SegmentEvent): SegmentEvent | undefined {
    return event;
  }

  /*
      Reset is called on an edge function when the system is told to reset.  Any
      user information should be discarded.
      */
  reset(): void {}

  /*
      Flush is called on an edge function when the system is told to send any
      queued events off to Segment.
      */
  flush(): void {}
}
